@@ -53,7 +53,7 @@ head_line <- function(function_name) {
 #' @description correct_get_color. (Version 0.0.0.9000)
 #' @param str taxa string
 #' @return a vector of colors
-#' @import dplyr
+#' @importFrom magrittr %>%
 #' @export
 correct_get_color <- function(str) {
   str <- as.factor(str)
@@ -406,7 +406,7 @@ get_color <- function(n=0,set="table2itol") {
 #' @description Paste rows group by key column
 #' @param df input data frame
 #' @return a data frame with pasted row by same id
-#' @import dplyr
+#' @importFrom magrittr %>%
 #' @export
 unite_rows <- function(df) {
   names <- names(df)
@@ -503,7 +503,14 @@ search_tree_file <- function(dir=getwd(),
   df <- file.info(results)
   df$fname <- file_get_name(rownames(df))
   if(method!='character'){
-    suppressWarnings(df <- df %>% arrange_(method))
+    suppressWarnings({
+      if (!method %in% colnames(df)) {
+        warning("method column not found; skipping sort")
+      } else {
+        ord <- order(df[[method]])
+        df <- df[ord, , drop = FALSE]
+      }
+    })
   }
   if(n == 'first'){
     df <- df %>% head(1)
@@ -513,3 +520,106 @@ search_tree_file <- function(dir=getwd(),
   }
   return(rownames(df))
 }
+
+#' Correct type parameter
+#'
+#' @description Correct type parameter input by string similarity and synonym mapping.
+#' @param str A character string representing a type name to correct.
+#' @importFrom stringdist stringsim
+#' @return A character string of the corrected type name.
+#' @export
+#' @examples
+#' correct_type("line")
+correct_type <- function(str) {
+    vec <- c('COLLAPSE', 'PRUNE', 'SPACING', 'TREE_COLORS', 'DATASET_STYLE', 'LABELS',
+             'DATASET_TEXT', 'DATASET_COLORSTRIP', 'DATASET_BINARY', 'DATASET_GRADIENT',
+             'DATASET_HEATMAP', 'DATASET_SYMBOL', 'DATASET_EXTERNALSHAPE', 'DATASET_DOMAINS',
+             'DATASET_SIMPLEBAR', 'DATASET_MULTIBAR', 'DATASET_BOXPLOT', 'DATASET_LINECHART',
+             'DATASET_PIECHART', 'DATASET_ALIGNMENT', 'DATASET_CONNECTIONS', 'DATASET_IMAGE', 'POPUP_INFO')
+    if(str %in% vec){
+      return(str)
+    }else{
+
+    input_str_lower <- tolower(str)
+
+    vec_lower <- tolower(vec)
+    vec_lower_no_prefix <- gsub("^dataset_", "", vec_lower)
+
+    best_match <- NULL
+    best_match_score <- -1
+
+    for (i in 1:length(vec)) {
+      target_str <- vec[i]
+      target_str_no_prefix <- vec_lower_no_prefix[i]
+
+      match_score_with_prefix <- longest_continuous_match(input_str_lower, target_str)
+      match_score_no_prefix <- longest_continuous_match(input_str_lower, target_str_no_prefix)
+
+      if (match_score_with_prefix > best_match_score) {
+        best_match_score <- match_score_with_prefix
+        best_match <- target_str
+      }
+      if (match_score_no_prefix > best_match_score) {
+        best_match_score <- match_score_no_prefix
+        best_match <- target_str
+      }
+    }
+
+    return(best_match)
+
+    }
+}
+
+#' Calculate longest continuous match between input and target strings
+#'
+#' @description This function calculates the longest continuous matching substring between an input string and a target string.
+#' @param input_str A character string to match against the target string.
+#' @param target_str A character string to compare with the input string.
+#' @return An integer value representing the longest continuous match.
+#' @export
+longest_continuous_match <- function(input_str, target_str) {
+  max_match_length <- 0
+  input_str <- gsub("[^a-zA-Z]", "", input_str)
+  target_str <- gsub("[^a-zA-Z]", "", target_str)
+
+  for (i in 1:(nchar(target_str) - nchar(input_str) + 1)) {
+    substring_target <- substr(target_str, i, i + nchar(input_str) - 1)
+
+    match_length <- 0
+    for (j in 1:nchar(input_str)) {
+      if (substr(input_str, j, j) == substr(substring_target, j, j)) {
+        match_length <- match_length + 1
+      } else {
+        break
+      }
+    }
+
+    max_match_length <- max(max_match_length, match_length)
+  }
+  return(max_match_length)
+
+}
+
+
+#' Transpose a data.table using the first column as new header
+#'
+#' Transpose a data.table such that the first column's values become the new column names,
+#' and the original column names (excluding the first) become a new 'Sample' column.
+#'
+#' @param dt A data.table to transpose. The first column contains the new column names.
+#' @return A transposed data.table with updated column names and a 'Sample' column.
+#' @importFrom data.table is.data.table transpose setnames setcolorder
+#' @export
+dt_transpose_header <- function(dt) {
+  if (!data.table::is.data.table(dt)) {
+    stop("Input must be a data.table.")
+  }
+  new_col_names <- dt[[1]]
+  transposed_dt <- data.table::transpose(dt[, -1, with = FALSE])
+  data.table::setnames(transposed_dt, new_col_names)
+  transposed_dt[, Sample := names(dt)[-1]]
+  data.table::setcolorder(transposed_dt, c("Sample", new_col_names))
+  return(transposed_dt)
+}
+
+
